@@ -2,7 +2,7 @@
 #
 # Copyright (C) 2005-2009 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
-# Copyright (C) 2006 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -30,7 +30,7 @@ __all__ = ['PatchRenderer']
 
 
 class PatchRenderer(Component):
-    """Structured display of patches in unified diff format.
+    """HTML renderer for patches in unified diff format.
 
     This uses the same layout as in the wiki diff view or the changeset view.
     """
@@ -40,19 +40,18 @@ class PatchRenderer(Component):
     # IHTMLPreviewRenderer methods
 
     def get_quality_ratio(self, mimetype):
-        if mimetype == 'text/x-diff':
+        if mimetype in ('text/x-diff', 'text/x-patch'):
             return 8
         return 0
 
     def render(self, context, mimetype, content, filename=None, rev=None):
         req = context.req
-        from trac.web.chrome import Chrome
-
         content = content_to_unicode(self.env, content, mimetype)
         changes = self._diff_to_hdf(content.splitlines(),
                                     Mimeview(self.env).tab_width)
-        if not changes:
-            raise TracError(_('Invalid unified diff content'))
+        if not changes or not any(c['diffs'] for c in changes):
+            self.log.warning('Invalid unified diff content')
+            return
         data = {'diff': {'style': 'inline'}, 'no_id': True,
                 'changes': changes, 'longcol': 'File', 'shortcol': ''}
 
@@ -75,10 +74,10 @@ class PatchRenderer(Component):
         If the diff cannot be parsed, this method returns None.
         """
         def _markup_intraline_change(fromlines, tolines):
-            from trac.versioncontrol.diff import _get_change_extent
+            from trac.versioncontrol.diff import get_change_extent
             for i in xrange(len(fromlines)):
                 fr, to = fromlines[i], tolines[i]
-                (start, end) = _get_change_extent(fr, to)
+                (start, end) = get_change_extent(fr, to)
                 if start != 0 or end != 0:
                     last = end+len(fr)
                     fromlines[i] = fr[:start] + '\0' + fr[start:last] + \
@@ -99,32 +98,57 @@ class PatchRenderer(Component):
         try:
             line = lines.next()
             while True:
-                if not line.startswith('--- '):
-                    if not line.startswith('Index: ') and line != '='*67:
-                        comments.append(line)
-                    line = lines.next()
-                    continue
-
                 oldpath = oldrev = newpath = newrev = ''
+                oldinfo = newinfo = []
+                binary = False
 
-                # Base filename/version
-                oldinfo = line.split(None, 2)
-                if len(oldinfo) > 1:
-                    oldpath = oldinfo[1]
-                    if len(oldinfo) > 2:
-                        oldrev = oldinfo[2]
+                # consume preample, storing free lines in comments
+                # (also detect the special case of git binary patches)
+                if not line.startswith('--- '):
+                    if not line.startswith('Index: ') and line != '=' * 67:
+                        comments.append(line)
+                    if line == "GIT binary patch":
+                        binary = True
+                        diffcmd_line = comments[0] # diff --git a/... b/,,,
+                        oldpath, newpath = diffcmd_line.split()[-2:]
+                        if any(c.startswith('new file') for c in comments):
+                            oldpath = '/dev/null'
+                        if any(c.startswith('deleted file') for c in comments):
+                            newpath = '/dev/null'
+                        oldinfo = ['', oldpath]
+                        newinfo = ['', newpath]
+                        index = [c for c in comments if c.startswith('index ')]
+                        if index: # index 8f****78..1e****5c
+                            oldrev, newrev = index[0].split()[-1].split('..')
+                            oldinfo.append(oldrev)
+                            newinfo.append(newrev)
+                        line = lines.next()
+                        while line:
+                            comments.append(line)
+                            line = lines.next()
+                    else:
+                        line = lines.next()
+                        continue
 
-                # Changed filename/version
-                line = lines.next()
-                if not line.startswith('+++ '):
-                    self.log.debug('expected +++ after ---, got '+line)
-                    return None
+                if not oldinfo and not newinfo:
+                    # Base filename/version from '--- <file> [rev]'
+                    oldinfo = line.split(None, 2)
+                    if len(oldinfo) > 1:
+                        oldpath = oldinfo[1]
+                        if len(oldinfo) > 2:
+                            oldrev = oldinfo[2]
 
-                newinfo = line.split(None, 2)
-                if len(newinfo) > 1:
-                    newpath = newinfo[1]
-                    if len(newinfo) > 2:
-                        newrev = newinfo[2]
+                    # Changed filename/version from '+++ <file> [rev]'
+                    line = lines.next()
+                    if not line.startswith('+++ '):
+                        self.log.debug('expected +++ after ---, got %s', line)
+                        return None
+
+                    newinfo = line.split(None, 2)
+                    if len(newinfo) > 1:
+                        newpath = newinfo[1]
+                        if len(newinfo) > 2:
+                            newrev = newinfo[2]
 
                 shortrev = ('old', 'new')
                 if oldpath or newpath:
@@ -142,6 +166,14 @@ class PatchRenderer(Component):
                         b = newpath[:-len(commonsuffix)]
                         if len(a) < 4 and len(b) < 4:
                             shortrev = (a, b)
+                    elif oldpath == '/dev/null':
+                        common = _("new file %(new)s",
+                                   new=newpath.lstrip('b/'))
+                        shortrev = ('-', '+')
+                    elif newpath == '/dev/null':
+                        common = _("deleted file %(deleted)s",
+                                   deleted=oldpath.lstrip('a/'))
+                        shortrev = ('+', '-')
                     else:
                         common = '(a) %s vs. (b) %s' % (oldpath, newpath)
                         shortrev = ('a', 'b')
@@ -149,9 +181,12 @@ class PatchRenderer(Component):
                     common = ''
 
                 groups = []
+                groups_title = []
                 changes.append({'change': 'edit', 'props': [],
                                 'comments': '\n'.join(comments),
+                                'binary': binary,
                                 'diffs': groups,
+                                'diffs_title': groups_title,
                                 'old': {'path': common,
                                         'rev': ' '.join(oldinfo[1:]),
                                         'shortrev': shortrev[0]},
@@ -161,16 +196,17 @@ class PatchRenderer(Component):
                 comments = []
                 line = lines.next()
                 while line:
-                    # "@@ -333,10 +329,8 @@" or "@@ -1 +1 @@"
-                    r = re.match(r'@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@',
-                                 line)
+                    # "@@ -333,10 +329,8 @@" or "@@ -1 +1 @@ [... title ...]"
+                    r = re.match(r'@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@'
+                                  '(.*)', line)
                     if not r:
                         break
                     blocks = []
                     groups.append(blocks)
-                    fromline, fromend, toline, toend = [int(x or 1)
-                                                        for x in r.groups()]
-                    last_type = last_change = extra = None
+                    fromline, fromend, toline, toend = \
+                            [int(x or 1) for x in r.groups()[:4]]
+                    groups_title.append(r.group(5))
+                    last_type = extra = None
 
                     fromend += fromline
                     toend += toline
@@ -184,7 +220,7 @@ class PatchRenderer(Component):
                         # Make a new block?
                         if (command == ' ') != last_type:
                             last_type = command == ' '
-                            kind = last_type and 'unmod' or 'mod'
+                            kind = 'unmod' if last_type else 'mod'
                             block = {'type': kind,
                                      'base': {'offset': fromline - 1,
                                               'lines': []},
@@ -205,8 +241,15 @@ class PatchRenderer(Component):
                             meta = block[last_side].setdefault('meta', {})
                             meta[len(block[last_side]['lines'])] = True
                             sides = [last_side]
+                        elif command == '@': # ill-formed patch
+                            groups_title[-1] = "%s (%s)" % (
+                                groups_title[-1],
+                                _("this hunk was shorter than expected"))
+                            line = '@'+line
+                            break
                         else:
-                            self.log.debug('expected +, - or \\, got '+command)
+                            self.log.debug('expected +, - or \\, got %s',
+                                           command)
                             return None
                         for side in sides:
                             if side == 'base':
