@@ -1,33 +1,63 @@
-# -*- encoding: utf-8 -*-
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2006-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.com/license.html.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/.
 
 import unittest
 
 from trac.test import Mock
-from trac.versioncontrol import NoSuchChangeset
 from trac.versioncontrol.api import *
 from trac.versioncontrol.web_ui import *
 from trac.wiki.tests import formatter
 
 
+YOUNGEST_REV = 200
+
+
 def _get_changeset(rev):
     if rev == '1':
-        return Mock(message="start", can_view=lambda perm: True)
+        return Mock(message="start", is_viewable=lambda perm: True)
     else:
         raise NoSuchChangeset(rev)
 
+
 def _normalize_rev(rev):
+    if rev is None or rev in ('', 'head'):
+        return YOUNGEST_REV
     try:
-        return int(rev)
+        nrev = int(rev)
+        if nrev <= YOUNGEST_REV:
+            return nrev
     except ValueError:
-        if rev == 'head':
-            return '200'
-        else:
-            raise NoSuchChangeset(rev)
-    
+        pass
+    raise NoSuchChangeset(rev)
+
+
+def _get_node(path, rev=None):
+    if path == 'foo':
+        return Mock(path=path, rev=rev, isfile=False,
+                    is_viewable=lambda resource: True)
+    elif path == 'missing/file':
+        raise NoSuchNode(path, rev)
+    else:
+        return Mock(path=path, rev=rev, isfile=True,
+                    is_viewable=lambda resource: True)
+
+
 def _get_repository(reponame):
-    return Mock(reponame=reponame, youngest_rev='200',
+    return Mock(reponame=reponame, youngest_rev=YOUNGEST_REV,
                 get_changeset=_get_changeset,
-                normalize_rev=_normalize_rev)
+                normalize_rev=_normalize_rev,
+                get_node=_get_node)
+
 
 def repository_setup(tc):
     setattr(tc.env, 'get_repository', _get_repository)
@@ -88,7 +118,13 @@ changeset:1#file0
 [1], r1
 </p>
 ------------------------------
-[1], r1
+============================== unicode digits
+[₁₂₃], r₁₂₃, [₀A₁B₂C₃D]
+------------------------------
+<p>
+[₁₂₃], r₁₂₃, [₀A₁B₂C₃D]
+</p>
+------------------------------
 ============================== Link resolver counter examples
 Change:[10] There should be a link to changeset [10]
 
@@ -109,8 +145,8 @@ trac:changeset:2081
 [trac:changeset:2081 Trac r2081]
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon">\xa0</span>trac:changeset:2081</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon">\xa0</span>Trac r2081</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon"></span>trac:changeset:2081</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon"></span>Trac r2081</a>
 </p>
 ------------------------------
 ============================== Changeset InterTrac shorthands
@@ -120,10 +156,10 @@ trac:changeset:2081
 T:r2081
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon">\xa0</span>[T2081]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon">\xa0</span>[trac 2081]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081/trunk" title="changeset:2081/trunk in Trac\'s Trac"><span class="icon">\xa0</span>[trac 2081/trunk]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/r2081" title="r2081 in Trac's Trac"><span class="icon">\xa0</span>T:r2081</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon"></span>[T2081]</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081" title="changeset:2081 in Trac's Trac"><span class="icon"></span>[trac 2081]</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/changeset%3A2081/trunk" title="changeset:2081/trunk in Trac\'s Trac"><span class="icon"></span>[trac 2081/trunk]</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/r2081" title="r2081 in Trac's Trac"><span class="icon"></span>T:r2081</a>
 </p>
 ------------------------------
 """ #"
@@ -141,6 +177,14 @@ LOG_TEST_CASES = u"""
 <a class="source" href="/log/trunk?revs=1-2">[2:1/trunk]</a> reversed, <a class="source" href="/log/trunk?revs=1-2">r2:1/trunk</a> reversed
 </p>
 ------------------------------
+============================== Big ranges (#9955 regression)
+[1234567890:12345678901]
+------------------------------
+<p>
+<a class="source" href="/log/?revs=1234567890-12345678901">[1234567890:12345678901]</a>
+</p>
+------------------------------
+<a class="source" href="/log/?revs=1234567890-12345678901">[1234567890:12345678901]</a>
 ============================== Escaping Log range TracLinks
 ![1:2], !r1:2, ![12:23], !r12:23
 ------------------------------
@@ -152,24 +196,47 @@ LOG_TEST_CASES = u"""
 ============================== log: link resolver
 log:@12
 log:trunk
+log:trunk@head
 log:trunk@12
 log:trunk@12:23
 log:trunk@12-23
 log:trunk:12:23
 log:trunk:12-23
+log:trunk@12:head
 log:trunk:12-head
-log:trunk:12@23 (bad, but shouldn't error out)
+log:trunk:12@23
 ------------------------------
 <p>
 <a class="source" href="/log/?rev=12">log:@12</a>
 <a class="source" href="/log/trunk">log:trunk</a>
+<a class="source" href="/log/trunk?rev=head">log:trunk@head</a>
 <a class="source" href="/log/trunk?rev=12">log:trunk@12</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk@12:23</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk@12-23</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk:12:23</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk:12-23</a>
-<a class="source" href="/log/trunk?revs=12-200">log:trunk:12-head</a>
-<a class="source" href="/log/trunk">log:trunk:12@23</a> (bad, but shouldn't error out)
+<a class="source" href="/log/trunk?revs=12%3Ahead">log:trunk@12:head</a>
+<a class="source" href="/log/trunk?revs=12-head">log:trunk:12-head</a>
+<a class="missing source" title="No changeset 12@23 in the repository">log:trunk:12@23</a>
+</p>
+------------------------------
+============================== log: link resolver with missing revisions
+log:@4242
+log:@4242-4243
+log:@notfound
+log:@deadbeef:deadbef0
+log:trunk@4243
+log:trunk@notfound
+[4242:4243]
+------------------------------
+<p>
+<a class="missing source" title="No changeset 4242 in the repository">log:@4242</a>
+<a class="source" href="/log/?revs=4242-4243">log:@4242-4243</a>
+<a class="missing source" title="No changeset notfound in the repository">log:@notfound</a>
+<a class="missing source" title="No changeset deadbeef in the repository">log:@deadbeef:deadbef0</a>
+<a class="missing source" title="No changeset 4243 in the repository">log:trunk@4243</a>
+<a class="missing source" title="No changeset notfound in the repository">log:trunk@notfound</a>
+<a class="source" href="/log/?revs=4242-4243">[4242:4243]</a>
 </p>
 ------------------------------
 ============================== log: link resolver + query
@@ -189,6 +256,21 @@ log:trunk@12?limit=10
 <a class="source" href="/log/trunk?revs=10-20&amp;verbose=yes&amp;format=changelog">[10:20/trunk?verbose=yes&amp;format=changelog]</a>
 </p>
 ------------------------------
+============================== log: link resolver + invalid ranges
+log:@10-20-30
+log:@10,20-30,40-50-60
+log:@10:20:30
+[10-20-30]
+[10:20:30]
+------------------------------
+<p>
+<a class="missing source" title="No changeset 10-20-30 in the repository">log:@10-20-30</a>
+<a class="missing source" title="No changeset 40-50-60 in the repository">log:@10,20-30,40-50-60</a>
+<a class="missing source" title="No changeset 10:20:30 in the repository">log:@10:20:30</a>
+[10-20-30]
+[10:20:30]
+</p>
+------------------------------
 ============================== Multiple Log ranges
 r12:20,25,35:56,68,69,100-120
 [12:20,25,35:56,68,69,100-120]
@@ -213,15 +295,28 @@ rfc:4180 should not be a log link
 [trac 3317:3318/trunk]
 ------------------------------
 <p>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/%403317%3A3318" title="log:/@3317:3318 in Trac\'s Trac"><span class="icon">\xa0</span>[T3317:3318]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/%403317%3A3318" title="log:/@3317:3318 in Trac\'s Trac"><span class="icon">\xa0</span>[trac 3317:3318]</a>
-<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/trunk%403317%3A3318" title="log:/trunk@3317:3318 in Trac\'s Trac"><span class="icon">\xa0</span>[trac 3317:3318/trunk]</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/%403317%3A3318" title="log:/@3317:3318 in Trac\'s Trac"><span class="icon"></span>[T3317:3318]</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/%403317%3A3318" title="log:/@3317:3318 in Trac\'s Trac"><span class="icon"></span>[trac 3317:3318]</a>
+<a class="ext-link" href="http://trac.edgewall.org/intertrac/log%3A/trunk%403317%3A3318" title="log:/trunk@3317:3318 in Trac\'s Trac"><span class="icon"></span>[trac 3317:3318/trunk]</a>
+</p>
+------------------------------
+============================== Log range with unicode digits
+r₁₂:₂₀,₂₅,₃₀-₃₅
+[₁₂:₂₀,₂₅,₃₀-₃₅]
+[T₃₃₁₇:₃₃₁₈]
+[trac ₃₃₁₇:₃₃₁₈]
+------------------------------
+<p>
+r₁₂:₂₀,₂₅,₃₀-₃₅
+[₁₂:₂₀,₂₅,₃₀-₃₅]
+[T₃₃₁₇:₃₃₁₈]
+[trac ₃₃₁₇:₃₃₁₈]
 </p>
 ------------------------------
 """
 
 
-DIFF_TEST_CASES = """
+DIFF_TEST_CASES = u"""
 ============================== diff: link resolver
 diff:trunk//branch
 diff:trunk@12//branch@23
@@ -252,7 +347,7 @@ diff://
 """
 
 
-SOURCE_TEST_CASES = """
+SOURCE_TEST_CASES = u"""
 ============================== source: link resolver
 source:/foo/bar
 source:/foo/bar#42   # no long works as rev spec
@@ -264,27 +359,29 @@ source:@42
 source:/foo/bar@42#L20
 source:/foo/bar@head#L20
 source:/foo/bar@#L20
+source:/missing/file
 ------------------------------
 <p>
-<a class="source" href="/browser/foo/bar">source:/foo/bar</a>
-<a class="source" href="/browser/foo/bar#42">source:/foo/bar#42</a>   # no long works as rev spec
-<a class="source" href="/browser/foo/bar#head">source:/foo/bar#head</a> #
-<a class="source" href="/browser/foo/bar?rev=42">source:/foo/bar@42</a>
-<a class="source" href="/browser/foo/bar?rev=head">source:/foo/bar@head</a>
-<a class="source" href="/browser/foo%2520bar/baz%252Bquux">source:/foo%20bar/baz%2Bquux</a>
-<a class="source" href="/browser/?rev=42">source:@42</a>
-<a class="source" href="/browser/foo/bar?rev=42#L20">source:/foo/bar@42#L20</a>
-<a class="source" href="/browser/foo/bar?rev=head#L20">source:/foo/bar@head#L20</a>
-<a class="source" href="/browser/foo/bar#L20">source:/foo/bar@#L20</a>
+<a class="source" href="/browser/foo/bar">source:/foo/bar</a><a class="trac-rawlink" href="/export/HEAD/foo/bar" title="Download"></a>
+<a class="source" href="/browser/foo/bar#42">source:/foo/bar#42</a><a class="trac-rawlink" href="/export/HEAD/foo/bar#42" title="Download"></a>   # no long works as rev spec
+<a class="source" href="/browser/foo/bar#head">source:/foo/bar#head</a><a class="trac-rawlink" href="/export/HEAD/foo/bar#head" title="Download"></a> #
+<a class="source" href="/browser/foo/bar?rev=42">source:/foo/bar@42</a><a class="trac-rawlink" href="/export/42/foo/bar" title="Download"></a>
+<a class="source" href="/browser/foo/bar?rev=head">source:/foo/bar@head</a><a class="trac-rawlink" href="/export/head/foo/bar" title="Download"></a>
+<a class="source" href="/browser/foo%2520bar/baz%252Bquux">source:/foo%20bar/baz%2Bquux</a><a class="trac-rawlink" href="/export/HEAD/foo%2520bar/baz%252Bquux" title="Download"></a>
+<a class="source" href="/browser/?rev=42">source:@42</a><a class="trac-rawlink" href="/export/42/" title="Download"></a>
+<a class="source" href="/browser/foo/bar?rev=42#L20">source:/foo/bar@42#L20</a><a class="trac-rawlink" href="/export/42/foo/bar#L20" title="Download"></a>
+<a class="source" href="/browser/foo/bar?rev=head#L20">source:/foo/bar@head#L20</a><a class="trac-rawlink" href="/export/head/foo/bar#L20" title="Download"></a>
+<a class="source" href="/browser/foo/bar#L20">source:/foo/bar@#L20</a><a class="trac-rawlink" href="/export/HEAD/foo/bar#L20" title="Download"></a>
+<a class="missing source">source:/missing/file</a>
 </p>
 ------------------------------
-============================== source: link resolver + query 
+============================== source: link resolver + query
 source:/foo?order=size&desc=1
 source:/foo/bar?format=raw
 ------------------------------
 <p>
 <a class="source" href="/browser/foo?order=size&amp;desc=1">source:/foo?order=size&amp;desc=1</a>
-<a class="source" href="/browser/foo/bar?format=raw">source:/foo/bar?format=raw</a>
+<a class="source" href="/browser/foo/bar?format=raw">source:/foo/bar?format=raw</a><a class="trac-rawlink" href="/export/HEAD/foo/bar" title="Download"></a>
 </p>
 ------------------------------
 ============================== source: provider, with quoting
@@ -294,10 +391,10 @@ source:"even with whitespaces"
 [source:"even with whitespaces" Path with spaces]
 ------------------------------
 <p>
-<a class="source" href="/browser/even%20with%20whitespaces">source:'even with whitespaces'</a>
-<a class="source" href="/browser/even%20with%20whitespaces">source:"even with whitespaces"</a>
-<a class="source" href="/browser/even%20with%20whitespaces">Path with spaces</a>
-<a class="source" href="/browser/even%20with%20whitespaces">Path with spaces</a>
+<a class="source" href="/browser/even%20with%20whitespaces">source:'even with whitespaces'</a><a class="trac-rawlink" href="/export/HEAD/even%20with%20whitespaces" title="Download"></a>
+<a class="source" href="/browser/even%20with%20whitespaces">source:"even with whitespaces"</a><a class="trac-rawlink" href="/export/HEAD/even%20with%20whitespaces" title="Download"></a>
+<a class="source" href="/browser/even%20with%20whitespaces">Path with spaces</a><a class="trac-rawlink" href="/export/HEAD/even%20with%20whitespaces" title="Download"></a>
+<a class="source" href="/browser/even%20with%20whitespaces">Path with spaces</a><a class="trac-rawlink" href="/export/HEAD/even%20with%20whitespaces" title="Download"></a>
 </p>
 ------------------------------
 ============================== export: link resolver
@@ -306,16 +403,16 @@ export:123:/foo/pict.gif
 export:/foo/pict.gif@123
 ------------------------------
 <p>
-<a class="export" href="/export/HEAD/foo/bar.html">export:/foo/bar.html</a>
-<a class="export" href="/export/123/foo/pict.gif">export:123:/foo/pict.gif</a>
-<a class="export" href="/export/123/foo/pict.gif">export:/foo/pict.gif@123</a>
+<a class="export" href="/export/HEAD/foo/bar.html" title="Download">export:/foo/bar.html</a>
+<a class="export" href="/export/123/foo/pict.gif" title="Download">export:123:/foo/pict.gif</a>
+<a class="export" href="/export/123/foo/pict.gif" title="Download">export:/foo/pict.gif@123</a>
 </p>
 ------------------------------
 ============================== export: link resolver + fragment
 export:/foo/bar.html#header
 ------------------------------
 <p>
-<a class="export" href="/export/HEAD/foo/bar.html#header">export:/foo/bar.html#header</a>
+<a class="export" href="/export/HEAD/foo/bar.html#header" title="Download">export:/foo/bar.html#header</a>
 </p>
 ------------------------------
 """ # " (be Emacs friendly...)
